@@ -120,7 +120,9 @@ impl<T> SetOps<T> for OrderedSet<T> where T: Copy {
         }
     }
 }
-    
+
+/// These are generally better than OrderedSet(s) for bulky end types, as
+/// there is not so much of moving them around.
 impl<T> SetOps<T> for IndexedSet<T> where T: Copy {
 
     /// just reverse the index
@@ -156,7 +158,7 @@ impl<T> SetOps<T> for IndexedSet<T> where T: Copy {
         else { memsearchdesc_indexed(&self.v,&self.i,m) }        
     }
     
-    /// Union of two indexed sets of the same type - 
+    /// Union of two IndexedSets. Returns an OrderedSet. 
     fn union(&self, s: &Self) -> OrderedSet<T> where T: PartialOrd { 
         if self.ascending {
             if s.ascending { OrderedSet{ ascending:self.ascending, v: unite_indexed(&self.v,&self.i,&s.v, &s.i)} }
@@ -183,5 +185,74 @@ impl<T> SetOps<T> for IndexedSet<T> where T: Copy {
         let s2 = OrderedSet::from_indexed(&s,true); 
         s1.difference(&s2) 
     }
+}
+
+/// For lots of set operations, it is probably better to work in IndexedSet(s) 
+/// and then only to rank the final result.
+impl<T> SetOps<T> for RankedSet<T> where T: Copy {
+
+    /// switches between ascending and descending ranks, 
+    /// which is what is logically expected here 
+    /// but it is not the same as a literal reversal of the ranks!
+    fn reverse(&self) -> Self where T: Copy {
+        Self { ascending: !self.ascending, v: self.v.to_vec(), i: self.i.complindex() }
+    }
         
+    /// Deletes repetitions.
+    fn nonrepeat(&self) -> Self where T: PartialOrd+Copy { 
+        let clean = &sansrepeat(&self.v);
+        Self { ascending: self.ascending, v: clean.to_vec(), i:rank(&clean, self.ascending )  }      
+    }
+     
+    /// Finds minimum, minimum's first index, maximum, maximum's first index
+    fn infsup(&self) -> (T, usize, T, usize) where T: PartialOrd {
+        let last = self.v.len()-1;
+        let si = self.i.invindex(); // ranks -> sort index
+        let firstval = self.v[si[0]];
+        let lastval = self.v[si[last]];
+            if self.ascending { (firstval,si[0],lastval,si[last]) }
+            else { (lastval,si[last],firstval,si[0]) }
+        }
+        
+    /// True if m is a member of the set
+    fn member(&self, m: T) -> bool where T: PartialOrd { 
+        let opt = if self.ascending { memsearch_indexed(&self.v,&self.i.invindex(),m) }
+                else { memsearchdesc_indexed(&self.v,&self.i.invindex(),m) };
+            opt.is_some()     
+        }
+        
+    /// Search a Set for m. Returns index of the first m.
+    fn search(&self, m: T)  -> Option<usize> where T: PartialOrd {
+        if self.ascending { memsearch_indexed(&self.v,&self.i.invindex(),m) }
+        else { memsearchdesc_indexed(&self.v,&self.i.invindex(),m) }        
+    }
+        
+    /// Union of two RankedSets. Returns an OrderedSet 
+    fn union(&self, s: &Self) -> OrderedSet<T> where T: PartialOrd { 
+        if self.ascending {
+            if s.ascending { OrderedSet{ ascending:self.ascending, v: unite_indexed(&self.v,&self.i.invindex(),&s.v, &s.i.invindex())} }
+            else { OrderedSet { ascending:true, v: unite_indexed(&self.v,  &self.i.invindex(),&s.v, &s.i.invindex().revindex() ) }     }
+        }
+        else {
+            if s.ascending { OrderedSet { ascending:true, v:unite_indexed(&self.v, &self.i.invindex().revindex(),&s.v,&s.i.invindex()) } }
+            else { OrderedSet { ascending:true, v:unite_indexed(&self.v, &self.i.invindex().revindex(), &s.v, &s.i.invindex().revindex()) } } 
+        }
+    }      
+        
+    /// Intersection of two RankedSets. 
+    /// Via OrderedSet for convenience, for now.
+    /// Todo: Probably should use intersect_indexed as in `union` above.
+    fn intersection(&self, s: &Self) -> OrderedSet<T> where T: PartialOrd {
+        let s1 = OrderedSet::from_ranked(&self,true);
+        let s2 = OrderedSet::from_ranked(&s,true);
+        s1.intersection(&s2)
+    }
+        
+    /// Complement of s in self (i.e. self-s)
+    fn difference(&self, s: &Self) -> OrderedSet<T> where T: PartialOrd {
+        let s1 = OrderedSet::from_ranked(&self,true);
+        let s2 = OrderedSet::from_ranked(&s,true); 
+        s1.difference(&s2) 
+    }        
+
 }
\ No newline at end of file
